#region Copyright
/*
 * Copyright 2005-2009 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
#endregion

using System.Collections.Generic;
using System.IO;
using System.Windows.Forms;
using AddInCommon.Util;
using EnvDTE;
using EnvDTE80;
using Microsoft.VisualStudio.CommandBars;
using VSArrange.Config;
using StatusBar = EnvDTE.StatusBar;
using VSArrange.Filter;

namespace VSArrange.Control
{
    /// <summary>
    /// \[VAvWFNgvf
    /// </summary>
    public class ArrangeControl
    {
        private const string REFRESH_BUTTON_NAME_SOLUTION = "SvWFNgvf̐";
        private const string REFRESH_BUTTON_NAME_PROJECT = "vWFNgvf̐";

        private readonly DTE2 _applicationObject;

        /// <summary>
        /// vWFNgڂƂȂt@C𔻕ʂ鐳K\
        /// </summary>
        private ItemAttachmentFilter _filterFile;

        /// <summary>
        /// t@Co^OtB^[
        /// </summary>
        public ItemAttachmentFilter FileterFile
        {
            set { _filterFile = value; }
            get { return _filterFile; }
        }

        /// <summary>
        /// vWFNgڂƂȂtH_𔻕ʂ鐳K\
        /// </summary>
        private ItemAttachmentFilter _filterFolder;

        /// <summary>
        /// tH_[o^OtB^[
        /// </summary>
        public ItemAttachmentFilter FilterFolder
        {
            set { _filterFolder = value; }
            get { return _filterFolder; }
        }
        
        /// <summary>
        /// RXgN^
        /// </summary>
        /// <param name="applicationObject"></param>
        public ArrangeControl(DTE2 applicationObject)
        {
            _applicationObject = applicationObject;
        }

        /// <summary>
        /// \[VENbNj[ɍڂǉĕԂ
        /// </summary>
        /// <param name="commandBar"></param>
        /// <returns></returns>
        public virtual CommandBarControl CreateSolutionContextMenuItem(CommandBar commandBar)
        {
            CommandBarButton refreshSolutuinButton =
                CommandBarUtils.CreateCommandBarControl<CommandBarButton>(commandBar);
            refreshSolutuinButton.Caption = REFRESH_BUTTON_NAME_SOLUTION;
            refreshSolutuinButton.Click += refreshSolutuinButton_Click;
            return refreshSolutuinButton;
        }

        /// <summary>
        /// vWFNgENbNj[ɍڂǉĕԂ
        /// </summary>
        /// <param name="commandBar"></param>
        /// <returns></returns>
        public virtual CommandBarControl CreateProjectContextMenuItem(CommandBar commandBar)
        {
            CommandBarButton refreshProjectButton =
                CommandBarUtils.CreateCommandBarControl<CommandBarButton>(commandBar);
            refreshProjectButton.Caption = REFRESH_BUTTON_NAME_PROJECT;
            refreshProjectButton.Click += refreshProjectButton_Click;
            return refreshProjectButton;
        }

        #region Cxg

        /// <summary>
        /// \[V{^NbNCxg
        /// </summary>
        /// <param name="Ctrl"></param>
        /// <param name="CancelDefault"></param>
        private void refreshSolutuinButton_Click(CommandBarButton Ctrl, ref bool CancelDefault)
        {
            Solution solution = _applicationObject.Solution;

            //  vWFNgǉtB^̍XV
            //  ݒ肪ύXꂽ_ŗ\ߔ񓯊œǂłǂ
            //  ptH[}XIɐOɓǂł肪ȂƎv邽
            //  Pɂ{RȂ߂ŌĂяo
            RefreshConfigInfo();
            try
            {
                foreach (Project project in solution.Projects)
                {
                    ArrangeProject(project);
                }
            }
            catch (System.Exception ex)
            {
                MessageBox.Show(ex.Message + "\n" + ex.StackTrace);
            }
            finally
            {
                StatusBarUtils.Clear(_applicationObject);
            }
        }

        /// <summary>
        /// vWFNg{^NbNCxg
        /// </summary>
        /// <param name="Ctrl"></param>
        /// <param name="CancelDefault"></param>
        private void refreshProjectButton_Click(CommandBarButton Ctrl, ref bool CancelDefault)
        {
            IDictionary<string, Project> refreshedProjects = new Dictionary<string, Project>();
            SelectedItems items = _applicationObject.SelectedItems;

            //  vWFNgǉtB^̍XV
            //  ݒ肪ύXꂽ_ŗ\ߔ񓯊œǂłǂ
            //  ptH[}XIɐOɓǂł肪ȂƎv邽
            //  Pɂ{RȂ߂ŌĂяo
            RefreshConfigInfo();
            try
            {
                foreach (SelectedItem selectedItem in items)
                {
                    Project currentProject = selectedItem.Project;
                    if (refreshedProjects.ContainsKey(currentProject.FullName))
                    {
                        //  XVς̃vWFNg͖
                        continue;
                    }

                    ArrangeProject(currentProject);
                    refreshedProjects[currentProject.FullName] = currentProject;
                }
            }
            catch (System.Exception ex)
            {
                MessageBox.Show(ex.Message + "\n" + ex.StackTrace);
            }
            finally
            {
                refreshedProjects.Clear();
                StatusBarUtils.Clear(_applicationObject);
            }
        }

        #endregion

        #region tbV

        /// <summary>
        /// vWFNg̃tbV
        /// </summary>
        /// <param name="project"></param>
        protected virtual void ArrangeProject(Project project)
        {
            if (string.IsNullOrEmpty(project.FullName))
            {
                //  vWFNgĂȂvf͖
                return;    
            }

            string projectDirPath = Path.GetDirectoryName(project.FullName) + Path.DirectorySeparatorChar;
            ProjectItems projectItems = project.ProjectItems;

            string statusLabel = string.Format("vWFNg[{0}]̗vf𐮗Ă܂B", project.Name);
            ArrangeDirectories(projectDirPath, projectItems, statusLabel);
        }

        /// <summary>
        /// fBNg̃tbV
        /// </summary>
        /// <param name="dirPath"></param>
        /// <param name="projectItems"></param>
        /// <param name="statusLabel"></param>
        protected virtual void ArrangeDirectories(string dirPath, ProjectItems projectItems, string statusLabel)
        {
            StatusBar bar = _applicationObject.StatusBar;

            IDictionary<string, ProjectItem> registeredItems = new Dictionary<string, ProjectItem>();
            //  폜Ăt@CAtH_vWFNgA[h
            for (int i = 1; i <= projectItems.Count; i++)
            {
                bar.Progress(true, statusLabel, i, projectItems.Count);
                
                ProjectItem projectItem = projectItems.Item(i);

                int beforeItemCount = projectItems.Count;
                string existPath = ArrangeItem(dirPath, projectItem, statusLabel);
                int afterItemCount = projectItems.Count;
                //  vWFNgvf폜ƃRNV̐
                //  ς邽߂̒
                if (beforeItemCount > afterItemCount)
                {
                    i = i - (beforeItemCount - afterItemCount);
                }

                if (existPath != null)
                {
                    registeredItems.Add(existPath, projectItem);
                }
            }

            LoadNotRegisterFile(dirPath, registeredItems, projectItems);
            LoadNotRegisterDirectory(dirPath, registeredItems, projectItems);
        }

        /// <summary>
        /// o^̃t@CvWFNgɒǉ
        /// </summary>
        /// <param name="dirPath"></param>
        /// <param name="registeredItems"></param>
        /// <param name="projectItems"></param>
        protected virtual void LoadNotRegisterFile(string dirPath,
                                                   IDictionary<string, ProjectItem> registeredItems,
                                                   ProjectItems projectItems)
        {
            string[] filePaths = Directory.GetFiles(dirPath);
            foreach (string s in filePaths)
            {
                string[] dirNamePathParts = s.Split('\\');
                string fileName = dirNamePathParts[dirNamePathParts.Length - 1];
                if (!registeredItems.ContainsKey(s) &&
                    _filterFile.IsPassFilter(fileName))
                {
                    projectItems.AddFromFile(s);
                }
            }
        }

        /// <summary>
        /// o^̃tH_vWFNgɒǉ
        /// </summary>
        /// <param name="dirPath"></param>
        /// <param name="registeredItems"></param>
        /// <param name="projectItems"></param>
        protected virtual void LoadNotRegisterDirectory(string dirPath,
                                                        IDictionary<string, ProjectItem> registeredItems,
                                                        ProjectItems projectItems)
        {
            string[] dirPaths = Directory.GetDirectories(dirPath);
            foreach (string s in dirPaths)
            {
                string[] dirNamePathParts = s.Split('\\');
                string dirName = dirNamePathParts[dirNamePathParts.Length - 1];
                if (!registeredItems.ContainsKey(s) &&
                    _filterFolder.IsPassFilter(dirName))
                {
                    ProjectItem addedProjectItem = projectItems.AddFromDirectory(s);
                    if (addedProjectItem != null)
                    {
                        //  璷ȂŌɒǉΏۊÕt@CAtH_΍폜
                        //  (AddFromDirectoryŏOΏۂ܂ŃvWFNgɊ܂܂Ă
                        //  \邽)
                        RemoveOutofTarget(addedProjectItem);
                    }
                }
            }
        }

        /// <summary>
        /// ΏۊOłvWFNgvfvWFNg珜O
        /// </summary>
        /// <param name="projectItem"></param>
        protected virtual void RemoveOutofTarget(ProjectItem projectItem)
        {
            foreach (ProjectItem item in projectItem.ProjectItems)
            {
                //  OtB^[Ɉꍇ̓vWFNg폜
                if(!_filterFile.IsPassFilter(item.Name) ||
                   !_filterFolder.IsPassFilter(item.Name))
                {
                    item.Remove();
                    continue;
                }

                if(item.ProjectItems.Count > 0)
                {
                    RemoveOutofTarget(item);
                }
            }
        }

        /// <summary>
        /// vf̍XVipX݂ȂvfvWFNg폜j
        /// </summary>
        /// <param name="dirPath"></param>
        /// <param name="item"></param>
        /// <param name="statusLabel"></param>
        protected virtual string ArrangeItem(string dirPath, ProjectItem item, string statusLabel)
        {
            string targetPath = dirPath + item.Name;
            if (File.Exists(targetPath) &&
                _filterFile.IsPassFilter(Path.GetFileName(targetPath)))
            {
                //  ݂t@CpX̏ꍇ͍폜ΏۂƂȂ
                return targetPath;
            }

            //  t@CłȂ΃tH_H
            if (Directory.Exists(targetPath) &&
                _filterFolder.IsPassFilter(Path.GetFileName(targetPath)))
            {
                string childDirName = targetPath + Path.DirectorySeparatorChar;
                //  ċAIɎqACe
                ArrangeDirectories(childDirName, item.ProjectItems, statusLabel);

                return targetPath;
            }
            //  t@CƂĂtH_ƂĂ݂ȂȂ
            //  vWFNgO
            item.Remove();
            return null;
        }

        #endregion

        /// <summary>
        /// ݒēǂݍ
        /// </summary>
        private void RefreshConfigInfo()
        {
            //  ݒǂݍ
            ConfigInfo configInfo = ConfigFileManager.ReadConfig(PathUtils.GetConfigPath());
            if (configInfo != null)
            {
                //  vWFNgvfǉOtB^[ݒ
                if (configInfo.FilterFileStringList != null)
                {
                    _filterFile.Clear();
                    _filterFile.AddFilters(configInfo.FilterFileStringList);
                }

                if (configInfo.FilterFolderStringList != null)
                {
                    _filterFolder.Clear();
                    _filterFolder.AddFilters(configInfo.FilterFolderStringList);
                }
            }
        }
    }
}